program GALILEE
  use sep
  use int2
  use interp

  implicit none
  integer                                       :: n2, nd, stat
  integer, dimension (2)                        :: m
  real,    dimension (:),   allocatable         :: z, mm, ones, count
  real,    dimension (:),   pointer             :: x, y
  real,    dimension (:,:), allocatable, target :: coord
  real,    dimension (2)                        :: o, d, min, max, cent, rad
  real                                          :: dx
  character (len=80)                            :: int

  call sep_init ()

  ! create coordinates
  call from_history (nd, n2)
  if (n2 /= 3) call erexit ('bad n2')
  allocate (coord (nd,2))
  call sep_read (coord)
  x => coord (:,1)
  y => coord (:,2)

  ! create model
  call from_par ("m1", m (1), 200)
  call from_par ("m2", m (2), 200)

  call to_history ("n1", m (1))
  call to_history ("n2", m (2))
  call to_history ("n3", 1)

  call from_par ("xmin", min (1), minval (x))
  call from_par ("ymin", min (2), minval (y))
  call from_par ("xmax", max (1), maxval (x))
  call from_par ("ymax", max (2), maxval (y))

  dx = sum ((max - min) / (m - 1)) * 0.5

  call from_par ("o1", o (1), min (1))
  call from_par ("o2", o (2), min (2))
  call from_par ("d1", d (1), dx)
  call from_par ("d2", d (2), dx)

  ! create ordinates
  allocate (z (nd))

  if (exist_file ("dat")) then
     cent = (max + min)/2
     rad  = ((max - min)/2) ** 2
     z = sqrt ((x - cent (1)) ** 2 / rad (1) + (y - cent (2)) ** 2 / rad (2))
     call to_history ("n1",   nd,"dat")
     call to_history ("n2",    1,"dat")
     call to_history ("n3",    1,"dat")
     call to_history ("esize", 4,"dat")
     call sep_write (z, "dat")
  else
     call sep_read (z)
  end if
  ! initialize interpolation
  call from_par ("interp",int)
  select case (int(1:4))
     case ("bin2")
     call int2_init (coord, o, d, m, bin2_int,  (/ 1, 1 /), nd)
     case ("bili")
     call int2_init (coord, o, d, m, lin2_int, (/ 2, 2 /), nd)
  case default
     call erexit ("unknown interpolation type, interp=?")
  end select
  call sep_close ()

  ! fill model
  allocate (mm (m (1) * m (2)))
  allocate (ones (nd), count (size (mm))) ; ones = 1.
  stat = int2_lop (.true.,.false.,mm,z)
  stat = int2_lop (.true.,.false.,count,ones)
  where (count > 0.) mm = mm / count
  call int2_close ()

  call sep_write (mm)

  deallocate (coord, z, mm, ones, count)
  call exit (0)
end program GALILEE



