program Fill
  !  <binned.H Fill.x rect > full.H
  !	Fill empty bins in 2-D by minimizing output of lowcut filter.
  !   binned.H has empty bins with zero values.
  !  nrect1    - integer [1]      halfwidth on 1-axis  (=1 means no smoothing);
  !  nrect2    - integer [1]      halfwidth on 2-axis  (=1 means no smoothing)
  !                     halfwidth is the width in sample points of
  !                     a rectangle which when convolved with itself
  !                     gives a triangle.
  !   rect1   -  float[nrect1/d1] halfwidth on 1-axis in physical units.
  !   rect2   -  float[nrect2/d2] halfwidth on 2-axis in physical units.
  use sep
  use igrad2
  use ruffen2
  use mask1
  use cgstep_mod
  use solver_mod

  implicit none
  integer                             :: niter, n12, method, stat
  integer, dimension (2)              :: n, nrect
  real,    dimension (2)              :: o, d, rect
  real,    dimension (:), allocatable :: bin, res
  real                                :: eps
  logical, dimension (:), pointer     :: mask 

  call sep_init ()
  call from_history (n (1), n (2)) ; n12 = n (1) * n (2)

  call from_history ("o1", o (1)) ; call from_history ("d1", d (1))
  call from_history ("o2", o (2)) ; call from_history ("d2", d (2))

  call from_par ("nrect1", nrect (1), 1) 
  call from_par ("nrect2", nrect (2), 1) 

  rect = nrect*d
  call from_par ("rect1", rect (1), rect (1))
  call from_par ("rect2", rect (2), rect (2))
  nrect = .5 + rect/d

  call to_history ("nrect1", nrect (1))
  call to_history ("nrect2", nrect (2))

  call from_par ("method", method, 0)
  call from_par ("niter",  niter, 25)
  call from_par ("eps",     eps,  1.)
  call sep_close 

  allocate (bin (n12), mask (n12), res (n12))
  call sep_read (bin) ; mask = (bin /= 0.) ; res = 0.

  call cgstep_init ()

  call ruffen2_init (nrect, n, eps) 
  select case (method)
  case (1)
     call solver (ruffen2_op,   cgstep, x = bin, dat = res, niter = niter, &
     verb = .true., mask = mask, res = res, x0 = bin)
  case (2)
     call mask1_init (mask)
     call solver (mask1_op,     cgstep, x = bin, dat = bin, niter = niter, &
     verb = .true., reg = ruffen2_op, nreg = n12, resm = res, x0 = bin)
  case (4)
     call igrad2_init (n (1), n (2))
     call mask1_init (mask)
     call solver (mask1_op,     cgstep, x = bin, dat = bin, niter = niter, &
     verb = .true., reg = ruffen2_op, nreg = n12, x0 = bin)
     stat = igradx_op (.false.,.false.,bin,res)
  case (5)
     call igrad2_init (n (1), n (2))
     call mask1_init (mask)
     call solver (mask1_op,     cgstep, x = bin, dat = bin, niter = niter, &
     verb = .true., prec = triangle2_op, nprec = n12, eps = eps, xp = res)
  case default
     call igrad2_init (n (1), n (2))
     call mask1_init (mask)
     call solver (mask1_op,     cgstep, x = bin, dat = bin, niter = niter, &
     verb = .true., prec = triangle2_op, nprec = n12, eps = eps, xp = res)
     stat = igradx_op (.false.,.false.,bin,res)
  end select
  call ruffen2_close ()
  call cgstep_close ()
  call sep_write (res)

  deallocate (bin, mask, res)
  call exit (0)
end program Fill

