program Trackify
  !  <triplets.H Trackify.x [thresh=  short=5] > quads.H
  !	Identify tracks.  Append a fourth column to matrix with track number.
  !	Output track number is zero location not predictable
  !	header output: output 98 percentile of track prediction error,	
  !			which could be used as input threshold thresh=
  !	short=5   Track number set zero for tracks shorter than "short".
  use sep
  use quantile_mod

  implicit none
  integer                             :: nd, three, short, is, id, track
  real,    dimension (:), allocatable :: x, y, z, pe
  real                                :: t, thresh
  integer, dimension (:), allocatable :: status, tr
  integer, parameter                  :: begintrk = 1, midtrack = 2, endtrack = 3, bad = 4

  call sep_init ()
  call from_history (nd,three)
  if (three /= 3) call erexit ('bad n2')

  allocate (x (nd), y (nd), z (nd), pe (nd), tr (nd), status (nd))

  call from_par ("short", short, 5)

  call sep_read (x)
  call sep_read (y)
  call sep_read (z)

  pe(1 )= 0.
  pe(nd)= 0.
  pe(2:nd-1) = sqrt ((x (1:nd-2) - 2*x (2:nd-1) + x (3:nd))**2 &
  +             (y (1:nd-2) - 2*y (2:nd-1) + y (3:nd))**2 ) 

  if (exist_file ("prederr")) then
     call to_history ("esize", 4 , "prederr")
     call to_history ("n1",   nd , "prederr")
     call to_history ("n2",    1 , "prederr")
     call sep_write (pe, "prederr")
  end if

  t = quantile (nd/2, pe)		! median prediction error

  call to_history ("Median_location_prediction_accuracy", t)

  call from_par ("tresh", thresh, t * 5.)
  t=thresh	                          ! where( pe(.) <t ) we are on a track.
  call to_history ("threshold_pred_err_for_splitting_into_tracks", t)

  status = bad
  where (pe (3:nd-2) < t)                        status (3:nd-2) = midtrack
  where (pe (3:nd-2) >= t .and. pe (4:nd-1) < t) status (3:nd-2) = begintrk
  where (pe (3:nd-2) >= t .and. pe (2:nd-3) < t) status (3:nd-2) = endtrack

  if (pe (2)  < t) then
     status (2) = midtrack
     status (1) = begintrk
  else 
     if (pe(3)<t) status (2) = begintrk
  end if

  if (pe (nd-1) < t) then
     status (nd-1) = midtrack
     status (nd) = endtrack 
  else 
     if (pe(nd-2) < t) status(nd-1) = endtrack
  end if

  do id= 1, nd  					!  eliminate short tracks.
     if (status(id) == begintrk) is = id
     if (status(id) == endtrack .and. short > id-is+1) status (is:id) = bad
  end do

  track = 0
  do id=1, nd 
     if (status(id) == bad) then
        tr (id) = 0
     else
        if (status(id) == begintrk) track = track + 1
        tr (id) = track
     end if
  end do

  call to_history ("tracks", track)
  call to_history ("points not on tracks are zeros", count (tr == 0))
  call to_history ("n1", 4)

  call sep_close()

  call sep_write (real (tr))
  call exit (0)
end program Trackify
