program Galilee
  use sep
  use interp
  use int2
  use igrad2 
  use trig
  use ruffen2
  use cgstep_mod
  use solver_mod

  implicit none
  integer                               :: n2, nd, niter, m12, inttype, regtype, stat
  integer, dimension (2)                :: m, nrect
  real,    dimension (:),   allocatable :: z, mm, ms, res
  real,    dimension (:,:), allocatable :: coord
  real,    dimension (2)                :: o, d, rect
  real                                  :: eps

  call sep_init ()

  ! create coordinates
  call from_history (nd, n2)
  if (n2 /= 3) call erexit ('bad n2')
  allocate (coord (nd,2))
  call sep_read (coord)

  ! create ordinates
  allocate (z (nd), res (nd))
  if (exist_file ("dat")) then 
     call sep_read (z, "dat")
  else
     call sep_read (z)
  end if

  ! create model
  call from_aux ("mod","n1", m (1))
  call from_aux ("mod","n2", m (2))

  call to_history ("n1", m (1))
  call to_history ("n2", m (2))
  call to_history ("n3", 1)
  m12 = m (1) * m (2)

  call from_aux ("mod", "o1", o (1))
  call from_aux ("mod", "o2", o (2))
  call from_aux ("mod", "d1", d (1))
  call from_aux ("mod", "d2", d (2))

  allocate (mm (m12), ms (m12))
  call sep_read (mm, "mod")

  ! initialize interpolation
  call from_par ("inttype",inttype)
  if (inttype == 1) then
     call int2_init (coord, o, d, m, bin2_int,  (/ 1, 1 /))
  else
     call int2_init (coord, o, d, m, lin2_int, (/ 2, 2 /))
  end if

  ! initialize regularization
  call from_par ("nrect1", nrect (1), 1) 
  call from_par ("nrect2", nrect (2), 1) 

  rect = nrect*d
  call from_par ("rect1", rect (1), rect (1))
  call from_par ("rect2", rect (2), rect (2))
  nrect = .5 + rect/d

  call to_history ("nrect1", nrect (1))
  call to_history ("nrect2", nrect (2))

  call from_par ("eps", eps, 1.)

  ! solver
  call from_par ("niter",niter,0)
  call from_par ("regtype",regtype)

  if (exist_file ("res")) then
     call to_history ("n1",   nd,"res")
     call to_history ("n2",    1,"res")
     call to_history ("n3",    1,"res")
     call to_history ("esize", 4,"res")
  end if
  call sep_close ()

  call cgstep_init ()
  select case (regtype)
  case (1)
     call igrad2_init (m (1), m (2)) 
     call solver (int2_op, cgstep, reg= igrad2_op, nreg= 2*m12, res= res, &
     dat= z, x= ms, x0= mm, niter= niter, verb= .true., resm = mm)
  case (2)
     call ruffen2_init (nrect, m, eps) 
     call solver (int2_op, cgstep, reg= ruffen2_op, nreg= m12, res= res, &
     dat= z, x= ms, x0= mm, niter= niter, verb= .true., resm = mm)
     call ruffen2_close ()
  case (3)
     call trig_init (m (1), m (2))
     call solver (int2_op, cgstep, prec= trig_op, nprec= m12, res= res, &
     dat= z, x= mm, niter= niter, verb= .true., eps = eps)
     call trig_close ()
  case default
  end select
  call cgstep_close ()
  call int2_close ()


  call sep_write (mm)
  if (exist_file ("res")) call sep_write (res, "res")

  deallocate (coord, z, res, mm, ms)
  call exit (0)
end program Galilee


